import sys

ARGS = [(['filenames'], {'nargs': '+'}),
        (['--data-source', '-d'], {'default': 'UM', 'choices': ['UM', 'MONC']})]


def main(args):
    if (sys.flags.interactive != 1) or not hasattr(QtCore, 'PYQT_VERSION'):
        from pyqtgraph.Qt import QtCore, QtGui
        from omnium.threed_viewer import MainWindow

        app = QtGui.QApplication(sys.argv)

        filenames = args.filenames
        window = MainWindow(filenames, args.data_source)
        window.show()
        sys.exit(app.exec_())



from pyqtgraph.Qt import QtGui
import pyqtgraph as pg

from omnium.viewers.data_displays import DataDisplayWindow


class ProfileWindow(DataDisplayWindow):
    name = 'Plot'
    title = 'Plot Display'
    accepts_multiple_cubes = True

    def setupGui(self):
        super(ProfileWindow, self).setupGui()
        self.resize(400, 300)
        self.plotWidget = pg.PlotWidget(self)

        lhs_hsplitter = QtGui.QSplitter()
        lhs_hsplitter.addWidget(self.plotWidget)

        self.main_layout.addWidget(lhs_hsplitter)
        self.plots = {}
        self.data = []

    def setCubes(self, cubes):
        # Check all cubes have the same time dimension.
        assert len(set([c.shape[0] for c in cubes])) == 1
        self.cubes = cubes
        self.cube = cubes[0]
        for cube in self.cubes:
            if cube.ndim != 4:
                msg = 'Two few or too many dims: {} - {}'.format(cube.name(), cube.ndim)
                raise Exception(msg)

        for cube in self.cubes:
            heights = cube.coord('level_height').points
            self.data.append((cube.name(), heights, cube.data.mean(axis=(2, 3))))
        self.update()

    def update(self):
        # self.plotWidget.clear()
        for name, heights, data in self.data:
            if name not in self.plots:
                plot = self.plotWidget.plot(data[self.time_index], heights)
                self.plots[name] = plot
            else:
                plot = self.plots[name]
                plot.setData(data[self.time_index], heights)



from omnium.viewers.data_displays.data_display_window import DataDisplayWindow
from omnium.viewers.data_displays.twod_window import TwodWindow
from omnium.viewers.data_displays.threed_window import ThreedWindow
from omnium.viewers.data_displays.plot_window import PlotWindow
from omnium.viewers.data_displays.profile_window import ProfileWindow
from omnium.viewers.data_displays.profile_contour_window import ProfileContourWindow



"""Shows useful information about a suite"""
import os
from logging import getLogger

from omnium.suite import Suite

logger = getLogger('omnium')

ARGS = []


def main(args):
    suite = Suite()
    suite.load(os.getcwd())

    logger.info('Suite: {}'.format(suite.name))
    logger.info('')
    logger.info('Suite info:')
    for line in suite.info_lines():
        logger.info(line)
    logger.info('')
    if suite.is_init:
        logger.info('Suite config:')
        for line in suite.suite_config_lines():
            logger.info(line)



ARGS = [(['--analysis', '-a'], {'help': 'Analysis to run'}),
        (['--run-type', '-d'], {'help': 'cycle/expt/[suite]', 'default': 'suite'}),
        (['expts'], {'nargs': '*', 'help': 'Experiment to analyze'}),
        (['--all'], {'help': 'Run all analysis', 'action': 'store_true'}),
        (['--filenames', '-n'], {'help': 'Filenames to run on'}),
        (['--force', '-f'], {'help': 'Force run', 'action': 'store_true'}),
        (['--interactive', '-i'], {'help': 'Run interactively', 'action': 'store_true'})]


def main(args):
    from omnium.run_control import RunControl
    run_control = RunControl(args.run_type, args.expts, args.force, args.interactive)
    run_control.gen_analysis_workflow()

    if args.all:
        run_control.run_all()
    elif args.analysis:
        run_control.run_analysis(args.analysis, args.filenames)



import numpy as np
from functools import partial
from collections import OrderedDict

from pyqtgraph.Qt import QtCore, QtGui
import pyqtgraph as pg

from omnium.viewers.data_displays import DataDisplayWindow


class TwodWindow(DataDisplayWindow):
    name = 'Slice'

    level_slider_changed = QtCore.pyqtSignal(int)
    level_slider2_changed = QtCore.pyqtSignal(int)

    orientations = ['xy', 'xz', 'yz']
    scales = ['image', 'level', 'time', 'level/time']
    colour_maps = OrderedDict([
        ('blue-white-red', {
            'pos': np.array([0.0, 0.5, 1.0]),
            'colour': np.array([[0, 0, 255, 255],
                               [255, 255, 255, 255],
                               [255, 0, 0, 255]], dtype=np.ubyte)}),
        ('black-red-yellow-white', {
            'pos': np.array([0.0, 0.33, 0.66, 1.0]),
            'colour': np.array([[0, 0, 0, 255],
                               [255, 0, 0, 255],
                               [255, 255, 0, 255],
                               [255, 255, 255, 255]], dtype=np.ubyte)}),
        ('black-orange-red', {
            'pos': np.array([0.0, 0.5, 1.0]),
            'colour': np.array([[0, 0, 0, 255],
                               [255, 128, 0, 255],
                               [255, 0, 0, 255]], dtype=np.ubyte)}),
        ('blue-red', {
            'pos': np.array([0.0, 0.5, 1.0]),
            'colour': np.array([[0, 0, 255, 255],
                               [128, 0, 128, 255],
                               [255, 0, 0, 255]], dtype=np.ubyte)})])

    def __init__(self, parent):
        super(TwodWindow, self).__init__(parent)
        self.level_index2 = 0
        self.orientation = 'xy'
        self.scale = 'image'
        self.colour_map = 'blue-white-red'
        self._min_max_cache = {}
        self.updateColourMap(self.colour_map, False)
        self.scaleSymmetrySetting = False

    def saveState(self):
        state = super(TwodWindow, self).saveState()
        state['level_index'] = self.level_index
        state['level_index2'] = self.level_index2
        state['orientation'] = self.orientation
        state['scale'] = self.scale
        state['colour_map'] = self.colour_map
        return state

    def loadState(self, state):
        super(TwodWindow, self).loadState(state)
        self.level_index = state['level_index']
        self.level_index2 = state['level_index2']
        self.orientation = state['orientation']
        self.scale = state['scale']
        self.colour_map = state['colour_map']
        self.updateColourMap(self.colour_map, False)

        self.level_slider_changed.emit(self.level_index)
        self.level_slider2_changed.emit(self.level_index2)

    def scaleSymmetry(self):
        self.scaleSymmetrySetting = self.scaleSymmetryAction.isChecked()
        self.update()

    def updateScale(self, scale):
        self.scale = scale
        if self.scale == 'image':
            pass
        elif self.scale == 'level':
            pass
        self.update()

    def updateOrientation(self, orientation):
        self.orientation = orientation
        if self.orientation == 'xy':
            self.level_slider.setRange(0, self.cube.shape[1] - 1)
        elif self.orientation == 'xz':
            self.level_slider.setRange(0, self.cube.shape[2] - 1)
        elif self.orientation == 'yz':
            self.level_slider.setRange(0, self.cube.shape[3] - 1)
        self.update()

    def updateColourMap(self, colour_map, update=True):
        self.colour_map = colour_map
        pos, colour = map(self.colour_maps[self.colour_map].get, ('pos', 'colour'))
        cmap = pg.ColorMap(pos, colour)
        self._lut = cmap.getLookupTable(0.0, 1.0, 256)
        if update:
            self.update()

    def setupGui(self):
        super(TwodWindow, self).setupGui()

        self.menuView = QtGui.QMenu(self.menubar)
        # self.menuFile.setObjectName(_fromUtf8("menuFile"))
        self.menuView.setTitle("View")
        self.menubar.addAction(self.menuView.menuAction())

        orientationGroup = QtGui.QActionGroup(self, exclusive=True)
        for orientation in self.orientations:
            checked = orientation == self.orientation
            orientationAction = QtGui.QAction(orientation, self, checkable=True, checked=checked)
            orientationAction.triggered.connect(partial(self.updateOrientation, orientation))
            orientationGroup.addAction(orientationAction)
            self.menuView.addAction(orientationAction)

        self.menuView.addSeparator()

        scaleGroup = QtGui.QActionGroup(self, exclusive=True)
        for scale in self.scales:
            checked = scale == self.scale
            scaleAction = QtGui.QAction('Scale on {}'.format(scale),
                                        self, checkable=True, checked=checked)
            scaleAction.triggered.connect(partial(self.updateScale, scale))
            scaleGroup.addAction(scaleAction)
            self.menuView.addAction(scaleAction)

        self.scaleSymmetryAction = QtGui.QAction('Symmetric scale',
                                                 self, checkable=True, checked=checked)
        self.scaleSymmetryAction.triggered.connect(self.scaleSymmetry)
        self.menuView.addAction(self.scaleSymmetryAction)

        cmapGroup = QtGui.QActionGroup(self, exclusive=True)
        self.menuView.addSeparator()
        colourMapMenu = self.menuView.addMenu('Colour map')
        for colour_map in self.colour_maps:
            checked = colour_map == self.colour_map
            cmapAction = QtGui.QAction(colour_map, self, checkable=True, checked=checked)
            cmapAction.triggered.connect(partial(self.updateColourMap, colour_map))
            cmapGroup.addAction(cmapAction)
            colourMapMenu.addAction(cmapAction)

        self._img = pg.ImageItem(border='w')
        self._img.setOpts(axisOrder='row-major')
        glw = pg.GraphicsLayoutWidget()
        view = glw.addViewBox()
        view.addItem(self._img)

        self.lhs_hsplitter = QtGui.QSplitter()
        self.lhs_hsplitter.addWidget(glw)

        self.main_layout.addWidget(self.lhs_hsplitter)

    def setLevelSliderValue(self, value):
        self.level_index = value
        self.update()

    def setLevelSlider2Value(self, value):
        self.level_index2 = value
        self.update()

    def setCube(self, cube):
        assert 3 <= cube.ndim <= 5
        self.cube = cube
        if self.cube.ndim >= 4:
            self.level_slider = QtGui.QSlider(QtCore.Qt.Vertical)
            self.level_slider_changed.connect(self.level_slider.setValue)
            self.level_slider.valueChanged.connect(self.setLevelSliderValue)
            self.level_slider.setRange(0, self.cube.shape[1] - 1)
            self.lhs_hsplitter.addWidget(self.level_slider)

        if self.cube.ndim == 5:
            self.level_slider2 = QtGui.QSlider(QtCore.Qt.Vertical)
            self.level_slider2_changed.connect(self.level_slider2.setValue)
            self.level_slider2.valueChanged.connect(self.setLevelSlider2Value)
            self.level_slider2.setRange(0, self.cube.shape[2] - 1)
            self.lhs_hsplitter.addWidget(self.level_slider2)

        self.update()

    def update(self):
        if self.cube.ndim == 3:
            data = self.cube[self.time_index].data
            self.setWindowTitle('{0}: {1:.2f}d'.format(self.cube.name(), self.time_days))
        elif self.cube.ndim == 4:
            if self.orientation == 'xy':
                height = self.cube.coord('level_height').points[self.level_index]
                data = self.cube[self.time_index, self.level_index].data
                self.setWindowTitle('{0}: {1:.2f}d, z: {2:.2f}m'.format(self.cube.name(),
                                                                        self.time_days,
                                                                        height))
            elif self.orientation == 'xz':
                y = self.cube.coord('grid_latitude').points[self.level_index]
                data = self.cube[self.time_index, :, self.level_index].data
                self.setWindowTitle('{0}: {1:.2f}d, y: {2:.2f}m'.format(self.cube.name(),
                                                                        self.time_days,
                                                                        y))
            elif self.orientation == 'yz':
                x = self.cube.coord('grid_longitude').points[self.level_index]
                data = self.cube[self.time_index, :, ::-1, self.level_index].data
                self.setWindowTitle('{0}: {1:.2f}d, x: {2:.2f}m'.format(self.cube.name(),
                                                                        self.time_days,
                                                                        x))
        elif self.cube.ndim == 5:
            data = self.cube[self.time_index, self.level_index, self.level_index2].data
            self.setWindowTitle('{0}: {1:.2f}d, {2:.2f}m'.format(self.cube.name(),
                                                                 self.time_days,
                                                                 height))

        kwargs = {}

        def get_key(*args):
            return ':'.join(map(str, args))

        if self.scale == 'image':
            min_max = data.min(), data.max()
        elif self.scale == 'level':
            key = get_key(self.time_index, 'all', self.orientation)
            min_max = self._get_min_max_cache(key)
            if not min_max:
                min_max = (self.cube[self.time_index].data.min(),
                           self.cube[self.time_index].data.max())
                self._min_max_cache[key] = min_max
        elif self.scale == 'time':
            key = get_key('all', self.level_index, self.orientation)
            min_max = self._get_min_max_cache(key)
            if not min_max:
                min_max = (self.cube[:, self.level_index].data.min(),
                           self.cube[:, self.level_index].data.max())
                self._min_max_cache[key] = min_max
        elif self.scale == 'level/time':
            key = get_key('all', 'all', 'all')
            min_max = self._get_min_max_cache(key)
            if not min_max:
                min_max = self.cube.data.min(), self.cube.data.max()
                self._min_max_cache[key] = min_max

        if self.scaleSymmetrySetting:
            abs_max = max(abs(min_max[0]), abs(min_max[1]))
            kwargs['levels'] = (-abs_max, abs_max)
        else:
            kwargs['levels'] = min_max

        self._img.setLookupTable(self._lut)
        self._img.setImage(data, **kwargs)

    def _get_min_max_cache(self, key):
        if key in self._min_max_cache:
            return self._min_max_cache[key]
        return None



"""Provide an easy way of getting analyzer classes by name"""
import os
import sys
from glob import glob
from collections import OrderedDict
import inspect
import imp
from logging import getLogger

import omnium
from omnium.analyzer import Analyzer

logger = getLogger('omnium')


def get_analysis_classes(cwd=None):
    """Discovery of analysis classes in a given directory

    Searches in dir cwd/analysis for python files.
    Loads all python files and any subclasses of Analyzer are returned in a dict."""
    logger.debug('loading analyzers from: {}'.format(cwd))
    if not cwd:
        cwd = os.getcwd()
    modules = []
    local_python_path = os.path.join(cwd, 'analysis')

    # Load the modules.
    if os.path.exists(local_python_path):
        for filename in sorted(glob(os.path.join(local_python_path, '*.py'))):
            module_name = os.path.splitext(os.path.basename(filename))[0]
            module = imp.load_source(module_name, filename)
            modules.append(module)

    current_module = sys.modules[__name__]
    modules.append(current_module)

    # Find subclasses of Analyzer
    analysis_classes = OrderedDict()
    for module in modules:
        for name, obj in inspect.getmembers(module):
            if inspect.isclass(obj) and issubclass(obj, Analyzer) and not obj == Analyzer:
                if obj.analysis_name:
                    logger.debug('loading from: {}, {}'.format(cwd, obj.analysis_name))
                    analysis_classes[obj.analysis_name] = obj
    return analysis_classes



from collections import OrderedDict
import importlib
from logging import getLogger

logger = getLogger('omnium')

commands = [
    'convert',
    'ls-analyzers',
    'run',
    'shell',
    'stash',
    'suite-info',
    'suite-init',
    'sync-mirror',
    'version',
    'viewer',
]

modules = OrderedDict()
for command in commands:
    command_name = 'cmds.' + command.replace('-', '_')
    try:
        modules[command] = importlib.import_module('omnium.' + command_name)
    except ImportError, e:
        logger.warn('Cannot load module {}'.format(command_name))
        logger.warn(e)



"""Converts all files given as command line args"""
import os
import re
from logging import getLogger

logger = getLogger('omnium')

ARGS = [(['filenames'], {'nargs': '+', 'help': 'Filename to convert'}),
        (['--converter', '-c'], {'help': 'Converter to use', 'default': 'ff2nc'}),
        (['--delete'], {'help': 'Delete after successful conversion', 'action': 'store_true'}),
        (['--allow-non-standard'], {'help': 'Allow conversion of non-standard files',
                                    'action': 'store_true'}),
        (['--overwrite'], {'help': 'Overwrite new file if already exists', 'action': 'store_true'}),
        (['--zlib'], {'help': 'Use compression', 'action': 'store_true'})]


def main(args):
    import iris
    from omnium.converters import CONVERTERS
    converter = CONVERTERS[args.converter](args.overwrite, args.delete,
                                           args.allow_non_standard, args.zlib)

    for filename in args.filenames:
        converter.convert(filename)



"""Gives info about stash variables"""
import os
import re
from logging import getLogger

logger = getLogger('omnium')

ARGS = [(['--search', '-s'], {'help': 'Search for stash var'}),
        (['--get-name', '-g'], {'help': 'Get name of stash var with code: e.g. 0,4'}),
        ]


def format_var(stash_var):
    return '{0:>3},{1:>4}: {2}'.format(stash_var[0][0], stash_var[0][1], stash_var[1])


def main(args):
    import omnium
    stash = omnium.Stash()
    if args.search:
        stash_vars = stash.search(args.search)
        for stash_var in stash_vars:
            print(format_var(stash_var))
    elif args.get_name:
        sec, item = args.get_name.split(',')
        print(stash[int(sec)][int(item)])
    else:
        print('Either --search or --get-name')



import os
from collections import OrderedDict as odict

from utils import get_git_info


class State(object):
    """State of the omnium project directory

    location, git_hash, git_status and conda_env.
    """
    def __init__(self):
        self.location = self._get_location()
        self.git_hash, self.git_status = get_git_info(self.location)
        self.conda_env = self._get_conda_env()

    def _get_location(self):
        return os.path.dirname(os.path.realpath(__file__))

    def _get_conda_env(self):
        return os.environ.get('CONDA_DEFAULT_ENV')



import os
import re
import pickle
from collections import OrderedDict
from logging import getLogger

from pyqtgraph.Qt import QtCore, QtGui
from omnium.viewers.data_displays import (TwodWindow, ThreedWindow,
                                          PlotWindow, ProfileWindow, ProfileContourWindow)
from omnium import Stash

import iris

logger = getLogger('omnium')

MAP_NAME_TO_CLASS = OrderedDict([
    ('Slice', TwodWindow),
    ('3D', ThreedWindow),
    ('Plot', PlotWindow),
    ('Profile', ProfileWindow),
    ('Contour', ProfileContourWindow),
])


class ViewerControlWindow(QtGui.QMainWindow):
    time_slider_changed = QtCore.pyqtSignal(int)

    def __init__(self, state, filenames):
        super(ViewerControlWindow, self).__init__()
        # super(ViewerControlWindow, self).__init__(None, QtCore.Qt.WindowStaysOnTopHint)
        self.filenames = filenames
        self.cubes = OrderedDict()
        self.time_index = 0
        self.wins = []
        self.stash = Stash()
        self.times = []

        self.setupGui()
        if not state and self.filenames:
            self.addVarItems(self.filenames)

        # self.time_slider.setRange(0, self.cube_list_viewer.cubes[0].shape[0])
        self.time_slider.setSingleStep(1)
        self.time_slider.setPageStep(15)
        self.time_slider.setTickInterval(60)
        self.time_slider.setTickPosition(QtGui.QSlider.TicksRight)
        self.time_slider.valueChanged.connect(self.setTimeSliderValue)
        self.time_slider_changed.connect(self.time_slider.setValue)

        self.timer = QtCore.QTimer(self)
        self.connect(self.timer, QtCore.SIGNAL("timeout()"), self.fwd)

        if state:
            self.loadState(state)

    def saveState(self):
        state = {}
        state['pos'] = self.pos()
        state['size'] = self.size()
        state['time_index'] = self.time_index
        state['filenames'] = self.filenames
        state['child_windows'] = []
        for win in self.wins:
            if win.isVisible():
                win_state = win.saveState()
                win_state['cubes'] = []
                if win.accepts_multiple_cubes:
                    for cube in win.cubes:
                        for stream in self.cubes.keys():
                            if cube in self.cubes[stream]:
                                win_state['cubes'].append((stream, cube.name()))
                else:
                    # import ipdb; ipdb.set_trace()
                    cube = win.cube
                    for stream in self.cubes.keys():
                        if cube in self.cubes[stream]:
                            win_state['cubes'].append((stream, cube.name()))

                state['child_windows'].append(win_state)

        filename = QtGui.QFileDialog.getSaveFileName(self, 'Save State',
                                                     os.getcwd(), selectedFilter='*.pkl')
        if filename:
            with open(filename, 'w') as f:
                pickle.dump(state, f)

    def loadState(self, state_filename=None):
        if not state_filename:
            filename = QtGui.QFileDialog.getOpenFileName(self, 'Load State',
                                                         os.getcwd(), 'State files (*.pkl)')

            if not filename:
                return
        filename = state_filename
        with open(filename, 'r') as f:
            state = pickle.load(f)

        self.move(state['pos'])
        self.resize(state['size'])
        self.filenames = state['filenames']
        self.time_index = state['time_index']

        self.time_slider_changed.emit(self.time_index)

        for win in self.wins:
            win.close()
        self.wins = []

        if self.filenames:
            self.cubes = OrderedDict()
            self.addVarItems(self.filenames)

        for win_state in state['child_windows']:
            print(win_state)
            cubes = []
            for stream, cube_name in win_state['cubes']:
                for cube in self.cubes[stream]:
                    if cube_name == cube.name():
                        cubes.append(cube)
                        break

            Cls = MAP_NAME_TO_CLASS[win_state['name']]
            if Cls.accepts_multiple_cubes:
                win = Cls(self)
                win.loadState(win_state)
                win.setupGui()
                win.setCubes(cubes)
                win.setTime(self.times[self.time_index])
                win.show()
                self.wins.append(win)
            else:
                win = Cls(self)
                win.loadState(win_state)
                win.setupGui()
                win.setCube(cubes[0])
                win.setTime(self.times[self.time_index])
                win.show()
                self.wins.append(win)

    def back(self):
        self.time_slider_changed.emit(self.time_index - 1)

    def pause(self):
        self.timer.stop()

    def play(self):
        timeout = int(self.ui_timeout.text())
        self.timer.start(timeout)

    def fwd(self):
        self.time_slider_changed.emit(self.time_index + 1)

    def go(self):
        self.time_slider_changed.emit(int(self.ui_time_index.text()))

    def setTimeSliderValue(self, value):
        self.time_index = value
        self.ui_time_index.setText(str(self.time_index))
        for win in self.wins:
            win.setTime(self.times[self.time_index])
            # win.time_index = self.time_index

        for win in self.wins:
            win.update()

    def launch(self):
        cubes = []
        for item in self.var_selector.selectedItems():
            data = item.data(0, QtCore.Qt.UserRole)
            cube = data.toPyObject()
            # cube = self.cubes[index]
            cubes.append(cube)

        cb = self.ui_displays
        # import ipdb; ipdb.set_trace()

        display_name = str(self.ui_displays.currentText())

        Cls = MAP_NAME_TO_CLASS[display_name]
        if Cls.accepts_multiple_cubes:
            win = Cls(self)
            win.setupGui()
            win.setCubes(cubes)
            win.setTime(self.times[self.time_index])
            win.show()
            self.wins.append(win)
        else:
            for cube in cubes:
                win = Cls(self)
                win.setupGui()
                win.setCube(cube)
                win.setTime(self.times[self.time_index])
                win.show()
                self.wins.append(win)

    def addCube(self, parent, cube):
        cube_name = ' '.join(cube.name().split())
        if cube.ndim < 2:
            logger.info('Cube {} only has {} dim(s) - not adding'.format(cube_name, cube.ndim))
            return

        nt = str(cube.shape[0])
        nx = str(cube.shape[-2])
        ny = str(cube.shape[-1])
        if cube.ndim == 4:
            nz = str(cube.shape[1])
        else:
            nz = ''
        cube_item = QtGui.QTreeWidgetItem(parent, [cube_name, nt, nz, nx, ny])
        cube_item.setData(0, QtCore.Qt.UserRole, cube)
        self.updateTimes(cube)

    def updateTimes(self, cube):
        new_times = cube.coord('time').points
        # import ipdb; ipdb.set_trace()
        if len(self.times) and (new_times[0] > self.times[-1] or new_times[-1] < self.times[0]):
            print('WARNING: times do not overlap')
        if len(new_times) > len(self.times):
            print('Using new times from {}: len {}'.format(cube.name(), len(new_times)))
            self.times = new_times
            self.time_slider.setRange(0, len(self.times) - 1)

    def matchFilename(self, filename):
        match = re.match('atmos.(?P<hours>\d{3}).(?P<stream>pp\d{1})(?P<ext>\.nc|)', filename)

        if match:
            return match.groups()
        else:
            match = re.match('atmos.(?P<stream>pp\d{1})(?P<ext>\.nc|)', filename)
            if match:
                return (None, match.group('stream'), match.group('ext'))
            else:
                return (None, 'default', '.nc')

    def addFile(self, filename):
        print(filename)
        fn = os.path.basename(filename)
        hours, stream, ext = self.matchFilename(fn)

        new_cubes = iris.load(filename)

        # self.stash.rename_unknown_cubes(new_cubes, True)

        if stream in self.cubes:
            self.cubes[stream].extend(new_cubes)
            self.cubes[stream] = self.cubes[stream].concatenate()
        else:
            self.cubes[stream] = new_cubes

    def addVarItems(self, filenames):
        for filename in filenames:
            self.addFile(filename)

        self.var_selector.clear()
        self.times = []

        root = self.var_selector.invisibleRootItem()
        for stream, cubes in self.cubes.items():
            stream_item = QtGui.QTreeWidgetItem(root, [stream])
            stream_item.setExpanded(True)
            for cube in cubes:
                self.addCube(stream_item, cube)

    def selectedItemChanged(self):
        print('selectedItemChanged')

    def handleVarSelectorChanged(self, item, column):
        pass

    def open(self):
        dlg = QtGui.QFileDialog(self, 'Open Cubes', os.getcwd())
        dlg.setFileMode(QtGui.QFileDialog.ExistingFiles)
        dlg.setFilter("Cube files (*.nc *.pp?)")
        dlg.setFilter
        filenames = []
        if dlg.exec_():
            for filename in dlg.selectedFiles():
                filenames.append(str(filename))
            print(filenames)
            self.filenames.extend(filenames)
            self.addVarItems(filenames)

    def setupGui(self):
        self.resize(400, 600)
        central_widget = QtGui.QWidget()

        self.menubar = QtGui.QMenuBar(self)
        self.menubar.setGeometry(QtCore.QRect(20, 20, 800, 23))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtGui.QMenu(self.menubar)
        self.menuFile.setTitle("File")
        self.menubar.addAction(self.menuFile.menuAction())
        self.setMenuBar(self.menubar)

        openAction = QtGui.QAction('&Open Cubes', self)
        openAction.setShortcut('Ctrl+O')
        openAction.triggered.connect(self.open)

        loadStateAction = QtGui.QAction('&Load State', self)
        loadStateAction.setShortcut('Ctrl+L')
        loadStateAction.setStatusTip('Load settings')
        loadStateAction.triggered.connect(self.loadState)

        saveStateAction = QtGui.QAction('&Save State', self)
        saveStateAction.setShortcut('Ctrl+S')
        saveStateAction.setStatusTip('Save settings')
        saveStateAction.triggered.connect(self.saveState)

        self.menuFile.addAction(openAction)
        self.menuFile.addAction(loadStateAction)
        self.menuFile.addAction(saveStateAction)

        play_controls = QtGui.QWidget()
        play_controls_layout = QtGui.QGridLayout()
        play_controls.setLayout(play_controls_layout)
        play_controls.setFixedHeight(100)

        ui_back = QtGui.QPushButton('Back')
        ui_pause = QtGui.QPushButton('Pause')
        ui_play = QtGui.QPushButton('Play')
        ui_fwd = QtGui.QPushButton('Fwd')

        self.ui_timeout = QtGui.QLineEdit(str(200))
        self.ui_time_index = QtGui.QLineEdit(str(0))
        ui_go = QtGui.QPushButton('Go')
        self.ui_loop = QtGui.QCheckBox('Loop')

        ui_back.clicked.connect(self.back)
        ui_pause.clicked.connect(self.pause)
        ui_play.clicked.connect(self.play)
        ui_fwd.clicked.connect(self.fwd)
        ui_go.clicked.connect(self.go)

        self.time_slider = QtGui.QSlider(QtCore.Qt.Horizontal)
        play_controls_layout.addWidget(self.time_slider, 0, 0, 1, 4)
        play_controls_layout.addWidget(ui_back, 1, 0)
        play_controls_layout.addWidget(ui_pause, 1, 1)
        play_controls_layout.addWidget(ui_play, 1, 2)
        play_controls_layout.addWidget(ui_fwd, 1, 3)
        play_controls_layout.addWidget(self.ui_timeout, 2, 0)
        play_controls_layout.addWidget(self.ui_time_index, 2, 1)
        play_controls_layout.addWidget(ui_go, 2, 2)
        play_controls_layout.addWidget(self.ui_loop, 2, 3)

        self.var_selector = QtGui.QTreeWidget()
        self.var_selector.setHeaderLabels(['Name', 'nt', 'nz', 'nx', 'ny'])
        self.var_selector.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
        self.var_selector.itemSelectionChanged.connect(self.selectedItemChanged)
        self.ui_displays = QtGui.QComboBox()

        for name in MAP_NAME_TO_CLASS.keys():
            self.ui_displays.addItem(name)

        ui_launch = QtGui.QPushButton('Launch')
        ui_launch.clicked.connect(self.launch)
        cube_widget = QtGui.QWidget()
        cube_widget_layout = QtGui.QVBoxLayout()
        cube_widget_layout.addWidget(self.var_selector)
        cube_widget_layout.addWidget(self.ui_displays)
        cube_widget_layout.addWidget(ui_launch)
        cube_widget.setLayout(cube_widget_layout)

        lhs_vsplitter = QtGui.QSplitter()
        lhs_vsplitter.setOrientation(QtCore.Qt.Vertical)
        lhs_vsplitter.addWidget(play_controls)
        lhs_vsplitter.addWidget(cube_widget)

        main_hsplitter = QtGui.QSplitter()
        main_hsplitter.addWidget(lhs_vsplitter)

        main_layout = QtGui.QHBoxLayout()
        main_layout.addWidget(main_hsplitter)
        central_widget.setLayout(main_layout)
        self.setCentralWidget(central_widget)



import sys
import logging


# Thanks: http://stackoverflow.com/a/287944/54557
class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'


# Thanks: # http://stackoverflow.com/a/8349076/54557
class ColourConsoleFormatter(logging.Formatter):
    '''Format messages in colour based on their level'''
    dbg_fmt = bcolors.OKBLUE + '%(levelname)-8s' + bcolors.ENDC + ': %(message)s'
    info_fmt = bcolors.OKGREEN + '%(levelname)-8s' + bcolors.ENDC + ': %(message)s'
    file_fmt = bcolors.HEADER + '%(levelname)-8s' + bcolors.ENDC + ': %(message)s'
    warn_fmt = bcolors.WARNING + '%(levelname)-8s' + bcolors.ENDC + ': %(message)s'
    err_fmt = (bcolors.FAIL + '%(levelname)-8s' + bcolors.ENDC + bcolors.BOLD +
               ': %(message)s' + bcolors.ENDC)

    def __init__(self, fmt="%(levelno)s: %(msg)s"):
        logging.Formatter.__init__(self, fmt)

    def format(self, record):
        # Save the original format configured by the user
        # when the logger formatter was instantiated
        format_orig = self._fmt

        # Replace the original format with one customized by logging level
        if record.levelno == logging.DEBUG:
            self._fmt = ColourConsoleFormatter.dbg_fmt
        elif record.levelno == logging.INFO:
            self._fmt = ColourConsoleFormatter.info_fmt
        elif record.levelno == logging.WARNING:
            self._fmt = ColourConsoleFormatter.warn_fmt
        elif record.levelno == logging.ERROR:
            self._fmt = ColourConsoleFormatter.err_fmt

        # Call the original formatter class to do the grunt work
        result = logging.Formatter.format(self, record)

        # Restore the original format configured by the user
        self._fmt = format_orig

        return result


def setup_logger(debug=False, colour=True, warn_stderr=False):
    '''Gets a logger. Sets up root logger ('omnium') if nec.'''
    root_logger = logging.getLogger('omnium')
    root_logger.propagate = False

    if getattr(root_logger, 'is_setup', False):
        # Stops log being setup for a 2nd time during ipython reload(...)
        root_logger.debug('Root logger already setup')
    else:
        fmt = '%(levelname)-8s: %(message)s'
        if colour:
            console_formatter = ColourConsoleFormatter(fmt)
        else:
            console_formatter = logging.Formatter(fmt)

        if debug:
            level = logging.DEBUG
        else:
            level = logging.INFO

        stdoutStreamHandler = logging.StreamHandler(sys.stdout)
        stdoutStreamHandler.setFormatter(console_formatter)
        stdoutStreamHandler.setLevel(level)

        root_logger.setLevel(level)
        root_logger.addHandler(stdoutStreamHandler)

        if warn_stderr:
            stderrStreamHandler = logging.StreamHandler(sys.stderr)
            stderrStreamHandler.setFormatter(logging.Formatter(fmt))
            stderrStreamHandler.setLevel(logging.WARNING)
            root_logger.addHandler(stderrStreamHandler)

        root_logger.is_setup = True

    return root_logger



import os
import subprocess as sp
from logging import getLogger

from omnium.omnium_errors import OmniumError

logger = getLogger('omnium')


class Syncher(object):
    "Provides means for syncing files from a remote host to a mirror"

    # 1st --exclude: must come *before* includes or e.g. .omnium/suite.conf will be downloaded.
    # 2nd --exclude: makes sure that only the filetypes asked for are downloaded.
    cmd_fmt = ("rsync -zuar{verbose} --exclude '.omnium/' {progress} {include} "
               "--exclude '*' --prune-empty-dirs {host}:{path}/ {dst_suite}")

    def __init__(self, suite, host=None, base_path='work/cylc-run', verbose=False):
        self.suite = suite
        self.host = host
        self.base_path = base_path
        self.verbose = 'v' if verbose else ''
        self.progress = '--progress' if verbose else ''
        # First include is necessary to make sure all dirs are included?
        # Gets configuration, python, shell scripts, info, cylc suite, and logs by default.
        self.includes = ['*/', '*.conf', '*.py', '*.sh', '*.info',
                         'suite*rc*', 'log*Z', 'log*.tar.gz']
        if self.suite.is_in_suite:
            if not host:
                if self.suite.suite_config['settings']['suite_type'] == 'mirror':
                    remote_config = self.suite.suite_config['remote']
                    self.host = remote_config['host']
                    logger.debug('Set host to: {}'.format(self.host))

    def add_exts(self, exts):
        "Add extensions to sync from remote host to mirror"
        self.includes.extend(['*' + ext for ext in exts])

    def _sync(self, suite_name, dst_suite):
        include = ' '.join(["--include '{}'".format(inc) for inc in self.includes])

        path = os.path.join(self.base_path, suite_name)
        cmd = self.cmd_fmt.format(verbose=self.verbose,
                                  progress=self.progress,
                                  include=include,
                                  host=self.host,
                                  path=path,
                                  dst_suite=dst_suite)

        logger.debug(cmd)
        sp.call(cmd, shell=True)

    def create(self, suite_name):
        "Creates a suite, filled with files from remote host"
        logger.info('Creating suite mirror: {}'.format(suite_name))
        self._sync(suite_name, suite_name)
        logger.info('Created')

    def sync(self):
        "Syncs a suite with files from remote host, must be used within a suite"
        if not self.suite.is_in_suite:
            raise OmniumError('Sync can only be used with a valid suite')
        if self.suite.suite_config['settings']['suite_type'] != 'mirror':
            raise OmniumError('Sync can only be used with a suite that is a mirror')

        remote_config = self.suite.suite_config['remote']
        if remote_config['host'] != self.host:
            msg_fmt = 'omnium config host different to requested host: {}, {}'
            logger.warn(msg_fmt.format(remote_config['host'], self.host))

        logger.info("Sync'ing suite mirror: {}".format(self.suite.name))

        logger.debug('cd to {}'.format(self.suite.suite_dir))
        cwd = os.getcwd()
        os.chdir(self.suite.suite_dir)

        self._sync(self.suite.name, '.')

        logger.debug('cd back to {}'.format(cwd))
        os.chdir(cwd)

        logger.info("Sync'd")



import os
import re
from collections import OrderedDict
from logging import getLogger

logger = getLogger('omnium')


class Stash(OrderedDict):
    def __init__(self):
        super(Stash, self).__init__()
        omnium_home = os.path.dirname(os.path.realpath(__file__))
        stash_master_path = os.path.join(omnium_home, 'data/files/vn10.5/STASHmaster_A')
        # stash_master_path = 'files/vn10.5/STASHmaster_A'

        with open(stash_master_path) as f:
            lines = f.readlines()

        for name_line in [l for l in lines if l[0] == '1']:
            split_line = [s.strip() for s in name_line.split('|')]
            section = int(split_line[2])
            if section not in self:
                self[section] = OrderedDict()
            item = int(split_line[3])
            name = split_line[4]
            self[section][item] = name

    def search(self, search):
        results = []
        for section, section_dict in self.items():
            for item, name in section_dict.items():
                if re.search(search, name, re.IGNORECASE):
                    results.append(((section, item), name))
        return results

    def get_name(self, section, item):
        return self[section][item]

    def rename_unknown_cubes(self, cubes, force=False):
        for cube in cubes:
            self.rename_unknown_cube(cube, force)

    def rename_unknown_cube(self, cube, force=False):
        if force or cube.name()[:7] == 'unknown':
            section = cube.attributes['STASH'].section
            item = cube.attributes['STASH'].item
            name = self[section][item]
            logger.info('Renaming {}->{}'.format(cube.name(), name))
            cube.rename(name)



import os
ARGS = []


def main(args):
    from omnium.suite import Suite

    suite = Suite()
    suite.load(os.getcwd())

    maxlen = max([len(k) for k in suite.analysis_classes.keys()])
    for k, v in suite.analysis_classes.items():
        fmt = '{0:' + str(maxlen) + '}: {1}'
        print(fmt.format(k, v))



import os
import io
from collections import OrderedDict
from logging import getLogger
from configparser import ConfigParser

from omnium.analyzers import get_analysis_classes
from omnium_errors import OmniumError
from utils import get_git_info

logger = getLogger('omnium')


class Suite(object):
    suite_types = ['runcontrol', 'run', 'archive', 'mirror']

    def __init__(self):
        self.name = None
        self.suite_dir = None
        self.is_in_suite = False
        self.is_omnium_app = False
        self.is_init = False
        self.local_analyzers_dir = None
        self.central_analyzers_dir = None
        self.analysis_classes = OrderedDict()

    def load(self, cwd):
        suite_dir = cwd
        while not os.path.exists(os.path.join(suite_dir, 'rose-suite.info')):
            suite_dir = os.path.dirname(suite_dir)
            if os.path.dirname(suite_dir) == suite_dir:
                # at root dir: /
                raise OmniumError('No rose-suite.info found - not in a suite?')
        self.is_in_suite = True
        self.suite_dir = suite_dir
        self.name = os.path.basename(suite_dir)
        logger.debug('in suite: {}'.format(self.suite_dir))

        # Check to see if it's already been initialized.
        config_filename = os.path.join(self.suite_dir, '.omnium/suite.conf')
        if os.path.exists(config_filename):
            self.is_init = True
            self.suite_config = ConfigParser()
            with open(config_filename, 'r') as f:
                self.suite_config.read_file(f)
            logger.debug('loaded suite config')

        # Check for omnium app.
        if os.path.exists(os.path.join(self.suite_dir, 'app/omnium/rose-app.conf')):
            self.is_omnium_app = True
            self.app_config_path = os.path.join(self.suite_dir, 'app/omnium/rose-app.conf')
            self.app_config = ConfigParser()
            with open(self.app_config_path, 'r') as f:
                self.app_config.read_file(f)
            # I have an app config. See if I can find central_analysis_classes:
            if 'OMNIUM_ANALYZERS_DIR' in self.app_config['env']:
                self.central_analyzers_dir = self.app_config['env']['OMNIUM_ANALYZERS_DIR']
            logger.debug('loaded app config')

        # Check for local analyzers.
        if os.path.exists(os.path.join(self.suite_dir, 'app/omnium/analysis')):
            self.local_analyzers_dir = os.path.join(self.suite_dir, 'app/omnium/')

        if self.local_analyzers_dir or self.central_analyzers_dir:
            # Load these first, these take precedence over centralized ones.
            if self.local_analyzers_dir:
                logger.debug('loading local analyzers')
                self.analysis_classes = get_analysis_classes(self.local_analyzers_dir)

            if self.central_analyzers_dir:
                logger.debug('loading central analyzers')
                git_hash, git_status = get_git_info(self.central_analyzers_dir)
                logger.debug('analyzers git_hash, status: {}, {}'.format(git_hash, git_status))
                central_analysis_classes = get_analysis_classes(self.central_analyzers_dir)
                self.central_analysis_hash = git_hash
                self.central_analysis_status = git_status
                # Add any analyzers *not already in classes*.
                for k, v in central_analysis_classes.items():
                    if k not in self.analysis_classes:
                        self.analysis_classes[k] = v

    def init(self, suite_type, host=None):
        assert suite_type in Suite.suite_types
        logger.info('Initializing suite, type: {}'.format(suite_type))
        logger.debug('Creating in {}'.format(self.suite_dir))
        self.load(self.suite_dir)

        suite_config = ConfigParser()
        suite_config.add_section('settings')
        suite_config.set('settings', 'suite_type', suite_type)

        if suite_type == 'mirror':
            suite_config.add_section('remote')
            suite_config.set('remote', 'host', host)

        dotomnium_dir = os.path.join(self.suite_dir, '.omnium')
        os.makedirs(dotomnium_dir)
        with open(os.path.join(dotomnium_dir, 'suite.conf'), 'wb') as configfile:
            suite_config.write(configfile)
        self.load(self.suite_dir)

    def suite_config_lines(self):
        conf_text = io.StringIO()
        self.suite_config.write(conf_text)
        conf_text.seek(0)
        return [l[:-1] for l in conf_text.readlines()]

    def info_lines(self):
        with open(os.path.join(self.suite_dir, 'rose-suite.info'), 'r') as f:
            return [l[:-1] for l in f.readlines()]



import logging
# Need to make sure that this is set to log something
# before everything else starts trying to get the logger.
logging.basicConfig()  # nopep8

from .version import __version__

from .omnium_cmd import main as omnium_main
from .stash import Stash


def init():
    """Useful for setting up variables in a script."""
    stash = Stash()
    return stash


def setup_ipython():
    """Injects useful variables into the global namespace. Only use interactively."""
    import sys
    from collections import OrderedDict
    import __main__ as main
    # Thanks: http://stackoverflow.com/a/2356420/54557
    # Guard to check being called interactively:
    if hasattr(main, '__file__'):
        raise Exception('Should only be used interactively')
    # Thanks: http://stackoverflow.com/a/14298025/54557
    builtin = sys.modules['__builtin__'].__dict__

    config, process_classes, dag, proc_eng, stash = init()
    globals_vars = OrderedDict()
    globals_vars['stash'] = stash
    print('Adding to global namespace:')
    for key, value in globals_vars.items():
        print('  ' + key)
        builtin[key] = value



from pyqtgraph.Qt import QtGui
import pyqtgraph as pg

from omnium.viewers.data_displays import DataDisplayWindow


class PlotWindow(DataDisplayWindow):
    name = 'Plot'
    title = 'Plot Display'
    accepts_multiple_cubes = True

    def setupGui(self):
        super(PlotWindow, self).setupGui()
        self.resize(400, 300)
        self.plotWidget = pg.PlotWidget(self)

        lhs_hsplitter = QtGui.QSplitter()
        lhs_hsplitter.addWidget(self.plotWidget)

        self.main_layout.addWidget(lhs_hsplitter)

    def setCubes(self, cubes):
        # Check all cubes have the same time dimension.
        assert len(set([c.shape[0] for c in cubes])) == 1
        self.cubes = cubes
        self.cube = cubes[0]

        for cube in cubes:
            if cube.ndim == 3:
                self.plotWidget.plot(range(cube.shape[0]), cube.data.mean(axis=(1, 2)))
            elif cube.ndim == 4:
                self.plotWidget.plot(range(cube.shape[0]), cube.data.mean(axis=(1, 2, 3)))
            elif cube.ndim == 5:
                self.plotWidget.plot(range(cube.shape[0]), cube.data.mean(axis=(1, 2, 3, 4)))
            else:
                msg = 'Two few or too many dims: {} - {}'.format(cube.name(), cube.ndim)
                raise Exception(msg)
        self.time_line = pg.InfiniteLine(movable=False, angle=90, label='t={value:0.2f}',
                                         labelOpts={'position': 0.1, 'color': (200, 200, 100),
                                                    'fill': (200, 200, 200, 50), 'movable': True})
        self.plotWidget.addItem(self.time_line)
        self.update()

    def update(self):
        self.time_line.setPos(self.time_index)



"""Initializes a new omnium suite"""
import os
from logging import getLogger

from omnium.suite import Suite

logger = getLogger('omnium')


ARGS = [(['--suite-type', '-t'], {'help': 'type of suite'}),
        (['--host'], {'help': 'mmuetz@login.archer.ac.uk', 'default': 'mmuetz@login.archer.ac.uk'})]


def main(args):
    suite = Suite()
    suite.load(os.getcwd())

    if args.suite_type not in Suite.suite_types:
        logger.error('--suite-type must be one of: {}'.format(Suite.suite_types))
        return 1
    if suite.is_init:
        logger.error('Suite already initialized')
        return 1

    suite.init(args.suite_type, args.host)
    logger.info('Suite initialized')



import os
import datetime as dt
from glob import glob
from collections import OrderedDict
import abc
from logging import getLogger

import iris

from omnium.omnium_errors import OmniumError
from omnium.version import get_version

logger = getLogger('omnium')


class Analyzer(object):
    __metaclass__ = abc.ABCMeta

    multi_file = False
    multi_expt = False

    @staticmethod
    def get_files(data_dir, filename):
        return sorted(glob(os.path.join(data_dir, filename)))

    def __init__(self, data_type, data_dir, results_dir,
                 filenames, expts):

        if self.multi_file and self.multi_expt:
            raise OmniumError('Only one of multi_file, multi_expt can be True')
        self.data_type = data_type
        self.data_dir = data_dir
        self.results_dir = results_dir
        if self.multi_expt:
            self.expt = None
            self.expts = expts
        else:
            assert len(expts) == 1
            self.expt = expts[0]
            self.expts = None

        if self.multi_file:
            self.filenames = filenames
            self.filename = None
            filename = os.path.basename(self.filenames[0])
        else:
            assert len(filenames) == 1
            filename = os.path.basename(filenames[0])
            if self.multi_expt:
                self.expt_filename = OrderedDict()
                for expt in self.expts:
                    self.expt_filename[expt] = os.path.join(self.data_dir[expt], filename)
                filename = os.path.basename(self.expt_filename.values()[0])
            else:
                self.filename = os.path.join(self.data_dir[self.expt], filename)
            self.filenames = None

        logger.debug('data_type: {}'.format(data_type))
        logger.debug('multi_file: {}'.format(self.multi_file))
        logger.debug('multi_expt: {}'.format(self.multi_expt))

        split_filename = filename.split('.')
        runid = split_filename[0]

        logger.debug('filename: {}'.format(filename))
        logger.debug('split_filename: {}'.format(split_filename))
        if data_type == 'datam':
            if len(split_filename) >= 3:
                time_hours = split_filename[1]
                instream = split_filename[2]
                if self.multi_file:
                    # TODO: v hacky nipping off last 3 chars.
                    # self.output_filename = '{}.{}.nc'.format(runid[:-3], self.analysis_name)
                    self.output_filename = '{}.{}.nc'.format(runid, self.analysis_name)
                else:
                    self.output_filename = '{}.{}.{}.nc'.format(runid,
                                                                time_hours,
                                                                self.analysis_name)
            elif len(split_filename) <= 2:
                logger.debug('Analyzing dump')
                # It's a dump. Should have a better way of telling though.
                if self.multi_file:
                    # TODO: hacky - nip off final 024, e.g. atmosa_da024 -> atmosa_da.
                    dump_without_time_hours = split_filename[0][:-3]
                    self.output_filename = '{}.{}.nc'.format(dump_without_time_hours,
                                                             self.analysis_name)
                else:
                    self.output_filename = '{}.{}.nc'.format(split_filename[0], self.analysis_name)
        elif data_type == 'dataw':
            instream = split_filename[1]
            self.output_filename = '{}.{}.nc'.format(runid, self.analysis_name)

        logger.debug('output_filename: {}'.format(self.output_filename))
        self.results = OrderedDict()
        self.force = False
        # N.B. there is only one results_dir, even for multi_expt
        self.logname = os.path.join(self.results_dir, self.output_filename + '.analyzed')

        # Need to make sure results dir exists before first call to self.append_log.
        if not os.path.exists(self.results_dir):
            logger.debug('Creating results_dir: {}'.format(self.results_dir))
            os.makedirs(self.results_dir)

    def set_config(self, config):
        logger.debug(config)
        for item in config.items():
            logger.debug(item)
        self._config = config
        self.force = self._config.getboolean('force', False)

    def already_analyzed(self):
        return os.path.exists(self.logname)

    def append_log(self, message):
        logger.debug('{}: {}'.format(self.analysis_name, message))
        with open(self.logname, 'a') as f:
            f.write('{}: {}\n'.format(dt.datetime.now(), message))

    def load(self):
        self.append_log('Loading')
        if self.multi_file:
            logger.debug('Loading {}'.format(self.filenames))
            self.cubes = iris.load(self.filenames)
        else:
            if self.multi_expt:
                self.expt_cubes = OrderedDict()
                for expt in self.expts:
                    logger.debug('Loading {}/{}'.format(expt, self.expt_filename[expt]))
                    self.expt_cubes[expt] = iris.load(self.expt_filename[expt])
            else:
                logger.debug('Loading {}'.format(self.filename))
                self.cubes = iris.load(self.filename)

        self.append_log('Loaded')

    def run(self, interactive=False):
        self.append_log('Analyzing')
        if interactive:
            logger.info('Running interactively')
            import ipdb
            ipdb.runcall(self.run_analysis)
        else:
            self.run_analysis()
        self.append_log('Analyzed')

    def save(self, state=None, suite=None):
        self.append_log('Saving')

        cubelist_filename = os.path.join(self.results_dir, self.output_filename)
        for cube_id, cube in self.results.items():
            logger.debug('Saving cube: {}'.format(cube.name()))
            logger.debug('omnium_cube_id: {}'.format(cube_id))
            logger.debug('cube shape: {}'.format(cube.shape))
            cube.attributes['omnium_vn'] = get_version('long')
            cube.attributes['omnium_cube_id'] = cube_id

            if state:
                cube.attributes['omnium_git_hash'] = state.git_hash
                cube.attributes['omnium_git_status'] = state.git_status
            if suite:
                cube.attributes['analyzers_git_hash'] = suite.central_analysis_hash
                cube.attributes['analyzers_git_status'] = suite.central_analysis_status

        cubelist = iris.cube.CubeList(self.results.values())
        if not len(cubelist):
            logger.warn('No results to save')
        else:
            logger.debug('Saving to {}'.format(cubelist_filename))
            # logger.debug('Not using zlib')
            # TODO: Make this a setting somewhere.
            # Use default compression: complevel 4.
            iris.save(cubelist, cubelist_filename, zlib=True)
            # iris.save(cubelist, cubelist_filename)

        self.save_analysis()
        self.append_log('Saved')

    def save_analysis(self):
        pass

    @abc.abstractmethod
    def run_analysis(self):
        return



import sys

ARGS = [(['filenames'], {'nargs': '*'}),
        (['--state', '-s'], {'nargs': '?', 'default': None}),
        (['--ignore-prev-settings'], {'action': 'store_true', 'default': False})]


def main(args):
    if (sys.flags.interactive != 1) or not hasattr(QtCore, 'PYQT_VERSION'):
        from pyqtgraph.Qt import QtCore, QtGui
        from omnium.viewers.viewer_control import ViewerControlWindow

        app = QtGui.QApplication(sys.argv)

        viewer_control = ViewerControlWindow(args.state, args.filenames)
        viewer_control.show()
        sys.exit(app.exec_())



import argparse
import argcomplete


def parse_commands(name, args, module, cmdline_args):
    # create the top-level parser.
    parser = argparse.ArgumentParser(prog=name)
    subparsers = parser.add_subparsers(dest='cmd_name')

    for args, kwargs in args:
        parser.add_argument(*args, **kwargs)

    cmds = module.modules
    for cmd_name, cmd_module in cmds.items():
        # create the subparser for each command.
        subparser = subparsers.add_parser(cmd_name,
                                          help=cmd_module.__doc__)
        for args, kwargs in cmd_module.ARGS:
            subparser.add_argument(*args, **kwargs)

    argcomplete.autocomplete(parser)
    args = parser.parse_args(cmdline_args)

    return cmds, args



"""Entry point into running omnium from command line

all commands are run as:
    omnium <cmd> <cmd_opts>
"""
import os
import sys

from command_parser import parse_commands
from setup_logging import setup_logger
from state import State
import cmds

# Top level args, e.g. omnium -D ...
ARGS = [(['--throw-exceptions', '-X'], {'action': 'store_true', 'default': False}),
        (['--DEBUG', '-D'], {'action': 'store_true', 'default': False}),
        (['--bw-logs', '-b'], {'action': 'store_true', 'default': False})]


def main(argv, import_log_msg=''):
    "Parse commands/env, setup logging, dispatch to cmd/<cmd>.py"
    omnium_cmds, args = parse_commands('omnium', ARGS, cmds, argv[1:])

    env_debug = os.getenv('OMNIUM_DEBUG') == 'True'
    cylc_control = os.getenv('CYLC_CONTROL') == 'True'
    omnium_dev = os.getenv('OMNIUM_DEV') == 'True'

    if args.DEBUG or env_debug:
        debug = True
    else:
        debug = False

    if cylc_control:
        colour = False
        warn_stderr = True
    else:
        colour = not args.bw_logs
        warn_stderr = False

    logger = setup_logger(debug, colour, warn_stderr)

    if omnium_dev:
        logger.info('running omnium_dev')
        import omnium
        logger.info(omnium)
    logger.debug('omnium import: {}'.format(import_log_msg))
    logger.debug(argv)
    logger.debug(args.cmd_name)
    logger.debug('cylc_control: {}'.format(cylc_control))

    state = State()
    logger.debug('omnium git_hash, status: {}, {}'.format(state.git_hash, state.git_status))

    cmd = omnium_cmds[args.cmd_name]
    if not args.throw_exceptions:
        logger.debug('Catching all exceptions')
        try:
            # dispatch on arg
            return cmd.main(args)
        except Exception as e:
            logger.error('{}'.format(e))
            return 1
    else:
        return cmd.main(args)



import numpy as np

from pyqtgraph.Qt import QtCore, QtGui
import pyqtgraph as pg

from omnium.viewers.data_displays import DataDisplayWindow


class ProfileContourWindow(DataDisplayWindow):
    name = 'Slice'

    def setupGui(self):
        super(ProfileContourWindow, self).setupGui()

        glw = pg.GraphicsLayoutWidget()
        self.view = glw.addViewBox()

        self.view.setAspectLocked()

        self.lhs_hsplitter = QtGui.QSplitter()
        self.lhs_hsplitter.addWidget(glw)

        self.main_layout.addWidget(self.lhs_hsplitter)

    def setCube(self, cube):
        assert cube.ndim == 4
        self.cube = cube

        data = self.cube.data.mean(axis=(2, 3))

        img = pg.ImageItem(data)
        self.view.addItem(img)

        # generate empty curves
        curves = []
        levels = np.linspace(data.min(), data.max(), 10)
        for i in range(len(levels)):
            v = levels[i]
            # generate isocurve with automatic color selection
            c = pg.IsocurveItem(level=v, pen=(i, len(levels)*1.5))
            c.setParentItem(img)  # make sure isocurve is always correctly displayed over image
            c.setZValue(10)
            curves.append(c)

        self.update()

    def update(self):
        pass



from pyqtgraph.Qt import QtCore, QtGui
import pyqtgraph.opengl as gl
import numpy as np

from omnium.viewers.data_displays import DataDisplayWindow


class ThreedWindow(DataDisplayWindow):
    name = '3D'
    title = '3D Display'
    accepts_multiple_cubes = True

    thresh_slider_changed = QtCore.pyqtSignal(float)
    neg_thresh_slider_changed = QtCore.pyqtSignal(float)
    size_slider_changed = QtCore.pyqtSignal(float)
    colours = [(1, 1, 1, 1),
               (0, 1, 1, 1),
               (0, 0, 1, 1),
               (1, 0, 1, 1),
               (1, 0.5, 1, 1),
               (1, 0, 0, 1),
               (1, 0.5, 0, 1),
               (1, 1, 0, 1)]

    def __init__(self, parent):
        super(ThreedWindow, self).__init__(parent)
        self.data_source = 'UM'
        self.timeout = 250
        self.cube_index = None

        self.rendered_point_scatters = {}
        self.point_scatters = {}
        self.cube_settings = {}
        self.cubes = []
        self.zn = None
        self.thresh = 0.
        self.neg_thresh = 0.

    def saveState(self):
        state = super(ThreedWindow, self).saveState()
        state['camera_position'] = tuple(self.view.cameraPosition())
        return state

    def loadState(self, state):
        super(ThreedWindow, self).loadState(state)

    def pick_colour(self):
        if self.cube_index is not None:
            colour = QtGui.QColorDialog.getColor().getRgbF()
            self.cube_settings[self.cube_index]['colour'] = colour

            self.point_scatters.pop(self.cube_index)
            self.point_scatters.pop(self.cube_index + 9999)
            self.add_remove()
            self.add_cube(self.cube_index)
            self.add_remove()

    def set_thresh_slider_value(self, value):
        frac = value / 1000.
        print(frac)
        if self.cube_index is not None and (self.cube_index in self.rendered_point_scatters or
                                            self.cube_index + 9999 in self.rendered_point_scatters):
            cube = self.cubes[self.cube_index]
            thresh = 20 * frac
            print(thresh)
            self.cube_settings[self.cube_index]['thresh'] = thresh

            self.point_scatters.pop(self.cube_index)
            self.point_scatters.pop(self.cube_index + 9999)
            self.add_remove()
            self.add_cube(self.cube_index)
            self.add_remove()
            self.ui_thresh.setText('{0:.3f}'.format(thresh))

    def set_neg_thresh_slider_value(self, value):
        frac = (1000 - value) / 1000.
        if self.cube_index is not None and (self.cube_index in self.rendered_point_scatters or
                                            self.cube_index + 9999 in self.rendered_point_scatters):
            cube = self.cubes[self.cube_index]
            neg_thresh = -20 * frac
            self.cube_settings[self.cube_index]['neg_thresh'] = neg_thresh

            self.point_scatters.pop(self.cube_index)
            self.point_scatters.pop(self.cube_index + 9999)
            self.add_remove()
            self.add_cube(self.cube_index)
            self.add_remove()
            self.ui_neg_thresh.setText('{0:.3f}'.format(neg_thresh))

    def set_size_slider_value(self, value):
        if self.cube_index is not None and self.cube_index in self.rendered_point_scatters:
            cube = self.cubes[self.cube_index]
            self.cube_settings[self.cube_index]['size_scale'] = value

            self.point_scatters.pop(self.cube_index)
            self.point_scatters.pop(self.cube_index + 9999)
            self.add_remove()
            self.add_cube(self.cube_index)
            self.add_remove()
            self.ui_size.setText('{0:.3f}'.format(value))

    def addCube(self, parent, cube):
        cube_name = ' '.join(cube.name().split())
        cube_item = QtGui.QTreeWidgetItem(parent, [cube_name])
        cube_item.setData(0, QtCore.Qt.UserRole, len(self.cubes))
        cube_item.setCheckState(0, QtCore.Qt.Unchecked)
        self.cubes.append(cube)

    def setCubes(self, cubes):
        # Check all cubes have the same time dimension.
        assert len(set([c.shape[0] for c in cubes])) == 1

        self.cube = cubes[0]
        root = self.var_selector.invisibleRootItem()
        for cube in cubes:
            assert cube.ndim == 4
            self.addCube(root, cube)

    def get_pos_size(self, cube, thresh, size_scale, neg=False):
        if neg:
            d = (cube.data < thresh)
        else:
            d = (cube.data > thresh)
        print(neg)
        print(cube.shape)

        # Get value of array where comparison is True.
        # size = cube.data[d] * multiplier
        size = 5
        if self.data_source == 'UM':
            # Needs some unpacking:
            # np.where(d) gets indices where comparison is true, but needs transposed.
            # indices are currently in order z, x, y, np.roll fixes this.
            pos_indices = np.roll(np.array(np.where(d)).T, -1, axis=1)

            # Map indices to values.
            pos = np.empty_like(pos_indices, dtype=np.float64)
            pos[:, 0] = -cube.coord('grid_longitude')\
                             .points[pos_indices[:, 0]] / 1000
            pos[:, 1] = cube.coord('grid_latitude')\
                            .points[pos_indices[:, 1]] / 1000
            pos[:, 2] = cube.coord('level_height')\
                            .points[pos_indices[:, 2]] / 1000

            pos -= [-128, 128, 0]
            print(pos.shape)
        elif self.data_source == 'MONC':
            # order is x, y, z
            pos_indices = np.array(np.where(d)).T

            # Map indices to values.
            pos = np.empty_like(pos_indices, dtype=np.float64)
            # TODO: x and y must have same dims currently.
            # TODO: these need to be calculated properly.
            xy_map = np.linspace(-(48 - 0.75), 48 - 0.75, cube.shape[0])
            # z_map = np.linspace(0, 40, cube.shape[-1])
            z_map = self.zn / 1000

            pos[:, 0] = xy_map[pos_indices[:, 0]]
            pos[:, 1] = xy_map[pos_indices[:, 1]]
            pos[:, 2] = z_map[pos_indices[:, 2]]

        return pos, size

    def update(self):
        print('updating')
        old_point_scatters_keys = self.point_scatters.keys()
        self.point_scatters = {}
        self.add_remove()
        for cube_index in old_point_scatters_keys:
            if cube_index < 9999:
                self.add_cube(cube_index)
        self.add_remove()

    def add_remove(self):
        for index in self.point_scatters.keys():
            if index not in self.rendered_point_scatters:
                print('adding {}'.format(index))
                point_scatter = self.point_scatters[index]
                self.view.addItem(point_scatter)
                self.rendered_point_scatters[index] = point_scatter

        for index in self.rendered_point_scatters.keys():
            if index not in self.point_scatters:
                print('removing {}'.format(index))
                point_scatter = self.rendered_point_scatters.pop(index)
                self.view.removeItem(point_scatter)

    def add_cube(self, cube_index):
        cube = self.cubes[cube_index]
        thresh = self.cube_settings[cube_index]['thresh']
        neg_thresh = self.cube_settings[cube_index]['neg_thresh']
        colour = self.cube_settings[cube_index]['colour']
        size_scale = self.cube_settings[cube_index]['size_scale']
        # cube_max = self.cube_settings[cube_index]['cube_max']

        pos, size = self.get_pos_size(cube[self.time_index], thresh, size_scale)
        point_scatter = gl.GLScatterPlotItem(pos=pos, color=(1, 0, 0, 1), size=size)
        point_scatter.setGLOptions('opaque')
        self.point_scatters[cube_index] = point_scatter

        pos, size = self.get_pos_size(cube[self.time_index], neg_thresh, size_scale, neg=True)
        point_scatter = gl.GLScatterPlotItem(pos=pos, color=(0, 0, 1, 1), size=size)
        point_scatter.setGLOptions('opaque')
        self.point_scatters[cube_index + 9999] = point_scatter

    def handleVarSelectorChanged(self, item, column):
        data = item.data(column, QtCore.Qt.UserRole)
        if item.checkState(column) == QtCore.Qt.Checked:
            index = data.toPyObject()
            cube = self.cubes[index]
            print(cube.name())
            print('checked')

            if index not in self.cube_settings:
                self.cube_settings[index] = {}
                colour = self.colours[index % len(self.colours)]
                self.cube_settings[index]['colour'] = colour
                self.cube_settings[index]['thresh'] = 1
                self.cube_settings[index]['neg_thresh'] = -1
                self.cube_settings[index]['size_scale'] = 5
                # self.cube_settings[index]['cube_max'] = cube.data.max()
            # Initial scatter.
            self.add_cube(index)
            self.add_remove()

        elif item.checkState(column) == QtCore.Qt.Unchecked:
            index = data.toPyObject()
            if index in self.point_scatters:
                self.point_scatters.pop(index)
                if index + 9999 in self.point_scatters:
                    self.point_scatters.pop(index + 9999)
                self.add_remove()

    def selectedItemChanged(self):
        item = self.var_selector.selectedItems()[0]
        data = item.data(0, QtCore.Qt.UserRole)
        index = data.toPyObject()
        self.cube_index = index
        cube = self.cubes[index]
        print(cube.name())
        self.var_info.setText(cube.__str__())
        if 'thresh' in self.cube_settings[index]:
            thresh = self.cube_settings[index]['thresh']
            # frac = thresh / cube.data.max()
            frac = thresh / 20
            value = 1000 * frac
            self.thresh_slider_changed.emit(value)
        if 'size_scale' in self.cube_settings[index]:
            size_scale = self.cube_settings[index]['size_scale']
            self.size_slider_changed.emit(size_scale)

    def setupGui(self):
        super(ThreedWindow, self).setupGui()

        self.resize(800, 600)

        self.var_selector = QtGui.QTreeWidget()
        self.var_selector.setHeaderLabels(["Tree"])
        self.var_selector.itemChanged.connect(self.handleVarSelectorChanged)
        self.var_selector.itemSelectionChanged.connect(self.selectedItemChanged)

        self.thresh_slider = QtGui.QSlider(QtCore.Qt.Horizontal)
        self.thresh_slider.setRange(0, 1000)
        self.thresh_slider.setSingleStep(1)
        self.thresh_slider.setTickInterval(1000)
        self.thresh_slider.setTickPosition(QtGui.QSlider.TicksRight)
        self.thresh_slider.valueChanged.connect(self.set_thresh_slider_value)
        self.thresh_slider_changed.connect(self.thresh_slider.setValue)
        self.thresh_slider.setValue(1000)

        self.neg_thresh_slider = QtGui.QSlider(QtCore.Qt.Horizontal)
        self.neg_thresh_slider.setRange(0, 1000)
        self.neg_thresh_slider.setSingleStep(1)
        self.neg_thresh_slider.setTickInterval(1000)
        self.neg_thresh_slider.setTickPosition(QtGui.QSlider.TicksRight)
        self.neg_thresh_slider.valueChanged.connect(self.set_neg_thresh_slider_value)
        self.neg_thresh_slider_changed.connect(self.neg_thresh_slider.setValue)
        self.neg_thresh_slider.setValue(1000)

        self.size_slider = QtGui.QSlider(QtCore.Qt.Horizontal)
        self.size_slider.setRange(0, 100)
        self.size_slider.setSingleStep(1)
        self.size_slider.setTickInterval(100)
        self.size_slider.setTickPosition(QtGui.QSlider.TicksRight)
        self.size_slider.valueChanged.connect(self.set_size_slider_value)
        self.size_slider_changed.connect(self.size_slider.setValue)
        self.size_slider.setValue(100)

        self.var_info = QtGui.QTextBrowser()
        self.var_info.setCurrentFont(QtGui.QFont('Monospace'))
        self.var_info.setFontPointSize(8)

        ui_pick_colour = QtGui.QPushButton('Pick Colour')
        ui_pick_colour.clicked.connect(self.pick_colour)

        self.ui_thresh = QtGui.QLabel()
        self.ui_thresh.setFixedWidth(50)
        self.ui_neg_thresh = QtGui.QLabel()
        self.ui_neg_thresh.setFixedWidth(50)
        self.ui_size = QtGui.QLabel()
        self.ui_size.setFixedWidth(50)

        lhs_bpanel = QtGui.QWidget()
        lhs_blayout = QtGui.QGridLayout()
        lhs_blayout.addWidget(self.thresh_slider, 0, 0)
        lhs_blayout.addWidget(self.ui_thresh, 0, 1)
        lhs_blayout.addWidget(self.neg_thresh_slider, 1, 0)
        lhs_blayout.addWidget(self.ui_neg_thresh, 1, 1)

        lhs_blayout.addWidget(self.var_info, 2, 0, 1, 2)
        lhs_bpanel.setLayout(lhs_blayout)
        lhs_vsplitter = QtGui.QSplitter()
        lhs_vsplitter.setOrientation(QtCore.Qt.Vertical)
        lhs_vsplitter.addWidget(self.var_selector)
        lhs_vsplitter.addWidget(lhs_bpanel)
        lhs_vsplitter.addWidget(ui_pick_colour)

        self.view = gl.GLViewWidget()
        self.view.opts['distance'] = 256
        # Add a grid.
        grid = gl.GLGridItem()
        # TODO: these need to be calculated properly.
        if self.data_source == 'MONC':
            grid.setSize(96, 96, 0)
            grid.setSpacing(4, 4, 0)

            zgrid = gl.GLGridItem()
            zgrid.setSize(96, 48, 0)
            zgrid.rotate(90, 1, 0, 0)
            zgrid.translate(0, -48, 24)
            zgrid.setSpacing(4, 4, 0)

            zgrid2 = gl.GLGridItem()
            zgrid2.setSize(48, 96, 0)
            zgrid2.rotate(90, 0, 1, 0)
            zgrid2.translate(-48, 0, 24)
            zgrid2.setSpacing(4, 4, 0)
        else:
            grid.setSize(256, 256, 0)
            grid.setSpacing(16, 16, 0)

            zgrid = gl.GLGridItem()
            zgrid.setSize(256, 32, 0)
            zgrid.rotate(90, 1, 0, 0)
            zgrid.translate(0, -128, 16)
            zgrid.setSpacing(16, 16, 0)

            zgrid2 = gl.GLGridItem()
            zgrid2.setSize(32, 256, 0)
            zgrid2.rotate(90, 0, 1, 0)
            zgrid2.translate(-128, 0, 16)
            zgrid2.setSpacing(16, 16, 0)

        self.view.addItem(grid)
        self.view.addItem(zgrid)
        self.view.addItem(zgrid2)

        self.ui_timeout = QtGui.QLineEdit(str(self.timeout))
        self.ui_time_index = QtGui.QLineEdit(str(self.time_index))
        ui_go = QtGui.QPushButton('Go')
        self.ui_loop = QtGui.QCheckBox('Loop')

        middle = QtGui.QWidget()
        middle_size_policy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        middle_size_policy.setHorizontalStretch(3)
        middle.setSizePolicy(middle_size_policy)

        middle_layout = QtGui.QVBoxLayout()

        middle.setSizePolicy(middle_size_policy)
        middle_layout.addWidget(self.view)
        middle.setLayout(middle_layout)

        main_hsplitter = QtGui.QSplitter()
        main_hsplitter.addWidget(lhs_vsplitter)
        main_hsplitter.addWidget(middle)
        self.main_layout.addWidget(main_hsplitter)



"""Launches IPython shell"""
import os

ARGS = [(['--failsafe'], {'action': 'store_true'})]


def main(args):
    import IPython

    if not args.failsafe:
        # Load up useful modules
        import iris
        import datetime as dt
        import numpy as np

        import omnium as om
        from omnium.omnium_errors import OmniumError
        from omnium.stash import Stash
        from omnium.state import State
        from omnium.suite import Suite
        from omnium.run_control import RunControl
        from omnium.converters import CONVERTERS
        from omnium.syncher import Syncher

        stash = Stash()
        state = State()
        suite = Suite()
        try:
            suite.load(os.getcwd())
        except OmniumError:
            print('NOT IN A SUITE')
            print('')

        import pylab as plt
    # IPython.start_ipython(argv=[])
    # This is better because it allows you to access e.g. args, config.
    IPython.embed()



import os
import re
import abc
from logging import getLogger
from itertools import izip

import numpy as np
import iris

from omnium.omnium_errors import OmniumError
from omnium.utils import get_cube

logger = getLogger('omnium')


class Converter(object):
    __metaclass__ = abc.ABCMeta

    def __init__(self, overwrite=False, delete=False, verify=False, allow_non_standard=False,
                 zlib=True):
        self.overwrite = overwrite
        self.delete = delete
        self.verify = verify
        self.allow_non_standard = allow_non_standard
        self.zlib = zlib

    def _converted_filename(self, old_filename):
        dirname = os.path.dirname(old_filename)
        filename = os.path.basename(old_filename)

        if not self.allow_non_standard:
            for pattern in self.standard_patterns:
                match = re.match(pattern, filename)

                if match:
                    logger.debug('matched {} with {}'.format(filename, pattern))
                    break

            if not match:
                raise OmniumError('Filename not standard: {}'.format(filename))

        newname = filename + '.nc'
        return os.path.join(dirname, newname)

    @abc.abstractmethod
    def _convert(self, filename, converted_filename):
        return

    def convert(self, filename):
        self.messages = ['archer_analysis convert ' + self.name]
        converted_filename = self._converted_filename(filename)
        logger.info('Convert: {} -> {}'.format(filename, converted_filename))

        if os.path.exists(converted_filename):
            if self.overwrite:
                logger.info('Deleting: {}'.format(converted_filename))
                self.messages.append('Deleting: {}'.format(converted_filename))
                os.remove(converted_filename)
            elif not os.path.exists(converted_filename + '.done'):
                # Could have been a problem creating file *or*
                # could still be being written.
                logger.info('Already converted but:')
                logger.warn('No .done file: {}'.format(converted_filename))
                return converted_filename
            else:
                logger.info('Already converted')
                return converted_filename

        self.messages.append('Original filename: {}'.format(filename))
        self.messages.append('New filename: {}'.format(converted_filename))

        self._convert(filename, converted_filename)

        if self.delete:
            logger.info('Delete: {}'.format(filename))
            os.remove(filename)
            self.messages.append('Deleted original')

        with open(converted_filename + '.done', 'w') as f:
            logger.debug('Writing ' + converted_filename + '.done')
            f.write('\n'.join(self.messages))
            f.write('\n')

        return converted_filename


class FF2NC(Converter):
    """Convert from Fields File (often .pp? extension) to NetCDF4 using iris"""
    name = 'ff2nc'
    standard_patterns = ['.*\.pp\d', '^atmosa_da(?P<ts>\d{3}$)']
    out_ext = '.nc'

    def _convert(self, filename, converted_filename):
        self.messages.append('Using iris to convert')
        cubes = iris.load(filename)
        if len(cubes) == 0:
            logger.warn('{} contains no data'.format(filename))
        else:
            logger.debug('Saving data to:{} (zlib={})'.format(converted_filename, self.zlib))
            # Use default compression: complevel 4.
            iris.save(cubes, converted_filename, zlib=self.zlib)

        if self.verify:
            logger.info('Verifying')
            # Check converted cubes are identical to originals.
            # Done in the _convert (i.e. derived impl.) because this way I don't have
            # to reload the data.
            converted_cubes = iris.load(converted_filename)
            for cube in cubes:
                logger.debug('Verify {}'.format(cube.name()))
                stash = cube.attributes['STASH']
                conv_cube = get_cube(converted_cubes, stash.section, stash.item)

                # Make cube iterators and slice over all but first dim.
                cube_it = cube.slices(range(1, cube.ndim))
                conv_cube_it = conv_cube.slices(range(1, conv_cube.ndim))

                # zip iterators together, grabbing slices and comparing them.
                for cslice, conv_cslice in izip(cube_it, conv_cube_it):
                    if not np.all(cslice.data == conv_cslice.data):
                        logger.error('Cubes not equal')
                        logger.error('Cube: {}'.format(cslice.name()))
                        logger.error('Time: {}'.format(cslice.coord('time')))
                        raise OmniumError('Mismatch in data between cubes')
            logger.info('Verified')


CONVERTERS = {
    FF2NC.name: FF2NC,
}



"""Print version info"""
import os
from omnium.version import get_version

ARGS = [(['-l', '--long'], {'help': 'print long version',
                            'action': 'store_true'})]


def main(args):
    if args.long:
        print('Version ' + get_version('long'))
    else:
        print('Version ' + get_version())



#!/usr/bin/env python
import os
import sys
try:
    from setuptools import setup, Extension
except ImportError:
    from distutils.core import setup, Extension

from omnium.version import get_version


def read(fname):
    try:
        return open(os.path.join(os.path.dirname(__file__), fname)).read()
    except:
        return ''


setup(
    name='omnium',
    version=get_version(),
    description='Output Management with Natural Interface for the UM',
    long_description=read('readme.rst'),
    author='Mark Muetzelfeldt',
    author_email='m.muetzelfeldt@pgr.reading.ac.uk',
    maintainer='Mark Muetzelfeldt',
    maintainer_email='m.muetzelfeldt@pgr.reading.ac.uk',
    packages=['omnium',
              'omnium.cmds',
              'omnium.data', 
              'omnium.viewers', 
              'omnium.viewers.data_displays'],
    scripts=[
        'bin/omnium',
        ],
    install_requires=[
        ],
    package_data={'omnium.data': ['files/vn10.5/STASHmaster*'],
        },
    url='https://github.com/markmuetz/omnium',
    classifiers=[
        'Environment :: Console',
        'Intended Audience :: Science/Research',
        'Natural Language :: English',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: C',
        'Topic :: Scientific/Engineering :: Atmospheric Science',
        ],
    keywords=[''],
    )



import os
import sys
import shutil
from glob import glob
from collections import OrderedDict
from logging import getLogger

from configparser import ConfigParser

from omnium.converters import CONVERTERS
from omnium.omnium_errors import OmniumError
from omnium.suite import Suite
from omnium.state import State

logger = getLogger('omnium')


class RunControl(object):
    def __init__(self, run_type, expts, force=False, interactive=False):
        self.cylc_control = os.getenv('CYLC_CONTROL') == 'True'
        self.run_type = run_type
        if self.run_type == 'suite':
            self.expts = expts
        else:
            assert len(expts) == 1
            self.expts = expts

        self.force = force
        self.interactive = interactive

        logger.warn('Disabling Python warnings')
        import warnings
        warnings.filterwarnings("ignore")
        self.setup()
        self.check_setup()

    def _read_env(self):
        suite_name = os.getenv('CYLC_SUITE_NAME')
        suite_base_dir = os.getenv('OMNIUM_BASE_SUITE_DIR')
        suite_dir = os.path.join(suite_base_dir, suite_name)

        cylc_suite_run_dir = os.getenv('CYLC_SUITE_RUN_DIR')
        if suite_dir != cylc_suite_run_dir:
            # TODO: HACKY!
            omnium_app_conf_path = 'app/omnium/rose-app.conf'
            src = os.path.join(cylc_suite_run_dir, omnium_app_conf_path)
            dst = os.path.join(suite_dir, omnium_app_conf_path)
            shutil.copyfile(src, dst)
            logger.debug('Copying omnium conf from {}'.format(src))
            logger.debug('                      to {}'.format(dst))

        initial_cycle_point = os.getenv('CYLC_SUITE_INITIAL_CYCLE_POINT')
        production = os.getenv('PRODUCTION') == 'True'

        return suite_name, initial_cycle_point, suite_dir, production

    def setup(self):
        suite = Suite()
        if self.cylc_control:
            # Running through cylc.
            (suite_name, initial_cycle_point, suite_dir, production) = self._read_env()
            try:
                suite.load(suite_dir)
            except OmniumError:
                logger.warn('Not a suite dir: {}'.format(suite_dir))
                return
        else:
            suite = Suite()
            production = False
            try:
                suite.load(os.getcwd())
            except OmniumError:
                logger.warn('Not in suite')
                return

        self.suite = suite
        suite_name = suite.name
        self.state = State()

        if production:
            logger.debug('running in production mode')
            if self.state.git_status != 'clean':
                raise OmniumError('omnium is not clean, not running')
            if suite.central_analysis_classes and suite.central_analysis_status != 'clean':
                raise OmniumError('omnium central analysis is not clean, not running')

        work_dir = os.path.join(self.suite.suite_dir, 'work')
        initial_cycle_point_dir = sorted(glob(os.path.join(work_dir, '*')))[0]
        initial_cycle_point = os.path.basename(initial_cycle_point_dir)
        suite_dir = suite.suite_dir
        self.analyzers_dir = self.suite.local_analyzers_dir

        self.suite_name = suite_name
        self.suite_dir = suite_dir

        self.atmos_datam_dir = {}
        self.atmos_dataw_dir = {}

        for expt in self.expts:
            self.atmos_datam_dir[expt] = os.path.join(suite_dir, 'share/data/history', expt)
            self.atmos_dataw_dir[expt] = os.path.join(suite_dir, 'work',
                                                      initial_cycle_point, expt + '_atmos')

    def check_setup(self):
        if self.analyzers_dir and not os.path.exists(self.analyzers_dir):
            logger.warn('Dir does not exist: {}'.format(self.analyzers_dir))
        for expt in self.expts:
            data_dir = self.atmos_datam_dir[expt]
            if not os.path.exists(data_dir):
                logger.warn('Dir does not exist: {}'.format(data_dir))
            data_dir = self.atmos_dataw_dir[expt]
            if not os.path.exists(data_dir):
                logger.warn('Dir does not exist: {}'.format(data_dir))

    def print_setup(self):
        for attr in ['cylc_control', 'run_type', 'expts', 'suite_name', 'analyzers_dir',
                     'atmos_datam_dir', 'atmos_dataw_dir']:
            print('{}: {}'.format(attr, getattr(self, attr)))

    def run(self):
        self.gen_analysis_workflow()
        self.run_all()

    def convert_all(self, converter_name, filename_globs, overwrite, delete):
        for expt in self.expts:
            if self.run_type == 'cycle':
                data_dir = self.atmos_datam_dir[expt]
            elif self.run_type == 'expt':
                data_dir = self.atmos_dataw_dir[expt]

            filenames = []
            logger.info('Converting files like:')
            for filename_glob in filename_globs:
                logger.info('  ' + filename_glob)
                for filename in sorted(glob(os.path.join(data_dir, filename_glob))):
                    # Make sure that UM has finished writing file.
                    if os.path.exists(filename + '.done'):
                        logger.debug('  Adding: {}'.format(filename))
                        filenames.append(filename)
                    else:
                        logger.warn('  Not finished: {}'.format(filename))

            converter = CONVERTERS[converter_name](overwrite, delete)
            filenames_for_conversion = []
            for filename in filenames:
                # Check that file is not being converted by another process.
                if os.path.exists(filename + '.converting'):
                    logger.warn('  Already being converted: {}'.format(filename))
                else:
                    with open(filename + '.converting', 'w') as f:
                        f.write('Converting with {}\n'.format(converter_name))
                    filenames_for_conversion.append(filename)

            if not filenames:
                logger.warn('No files to convert')

            for filename in filenames_for_conversion:
                try:
                    converter.convert(filename)
                    # N.B. if converter fails, there will be a left over .converting file.
                    # This is intentional: I want to see if this has failed.
                    os.remove(filename + '.converting')
                except OmniumError as oe:
                    logger.error('Could not convert {}'.format(filename))
                    logger.error(oe)

            if not filenames_for_conversion:
                logger.warn('No files not already being converted to convert')

    def gen_analysis_workflow(self):
        self.analysis_workflow = OrderedDict()

        config = self.suite.app_config
        suite_name = self.suite_name
        run_type = self.run_type
        expts = self.expts

        settings_sec = 'settings_{}'.format(run_type)
        runcontrol_sec = 'runcontrol_{}'.format(run_type)

        settings = config[settings_sec]

        convert = settings.getboolean('convert', False)
        if convert:
            converter_name = settings.get('converter', 'ff2nc')
            overwrite = settings.getboolean('overwrite', False)
            delete = settings.getboolean('delete', False)
            filename_globs = settings['filenames'].split(',')
            self.convert_all(converter_name, filename_globs, overwrite, delete)

        if runcontrol_sec in config:
            runcontrol = config[runcontrol_sec]
        else:
            logger.info('No runcontrol for {}'.format(self.run_type))
            return

        self.analysis_classes = self.suite.analysis_classes

        for ordered_analysis, enabled_str in sorted(runcontrol.items()):
            analysis = ordered_analysis[3:]
            enabled = enabled_str == 'True'
            # N.B. even if analyzer not enabled in config, want to make sure it
            # can still be run by e.g. run_analysis.
            if config.has_section(analysis):
                analyzer_config = config[analysis]
            else:
                raise OmniumError('NO CONFIG FOR ANALYSIS')

            if analysis not in self.analysis_classes:
                raise OmniumError('COULD NOT FIND ANALYZER: {}'.format(analysis))

            Analyzer = self.analysis_classes[analysis]
            logger.debug(Analyzer)

            filename_glob = analyzer_config['filename']
            logger.debug(filename_glob)

            logger.debug('Adding analysis: {}'.format(analysis))

            if analysis in self.analysis_workflow:
                raise OmniumError('{} already in analysis workflow'.format(analysis))

            self.analysis_workflow[analysis] = (Analyzer, analyzer_config['data_type'],
                                                analyzer_config, filename_glob, enabled)

        logger.debug(self.analysis_workflow.keys())

    def run_analysis(self, analysis, user_filename_glob=None):
        if not self.analysis_workflow:
            self.gen_analysis_workflow()
        (Analyzer, data_type, analyzer_config,
         filename_glob, enabled) = self.analysis_workflow[analysis]
        if user_filename_glob:
            filename_glob = user_filename_glob
            logger.debug('Using user defined glob: {}'.format(filename_glob))
        self._setup_run_analyzer(Analyzer, data_type, analyzer_config, filename_glob)

    def run_all(self):
        for (Analyzer, data_type, analyzer_config,
             filename_glob, enabled) in self.analysis_workflow.values():
            if enabled:
                self._setup_run_analyzer(Analyzer, data_type, analyzer_config, filename_glob)

    def _setup_run_analyzer(self, Analyzer, data_type, analyzer_config, filename_glob):
        logger.info('Running {} on {} files'.format(Analyzer.analysis_name, filename_glob))

        multi_file = Analyzer.multi_file
        multi_expt = Analyzer.multi_expt

        if data_type == 'datam':
            data_dir = self.atmos_datam_dir
        elif data_type == 'dataw':
            data_dir = self.atmos_dataw_dir

        if multi_file and multi_expt:
            raise OmniumError('Only one of multi_file, multi_expt can be True')

        if multi_expt:
            # N.B. multi_file == 'False'
            self._run_analyzer(Analyzer, data_type, analyzer_config,
                               [filename_glob], self.expts, multi_file, multi_expt)
        else:
            for expt in self.expts:
                filenames = Analyzer.get_files(data_dir[expt], filename_glob)
                if multi_file:
                    self._run_analyzer(Analyzer, data_type, analyzer_config,
                                       filenames, [expt], multi_file, multi_expt)
                else:
                    for filename in filenames:
                        self._run_analyzer(Analyzer, data_type, analyzer_config,
                                           [filename], [expt],
                                           multi_file, multi_expt)

    def _run_analyzer(self, Analyzer, data_type, analyzer_config,
                      filenames, expts, multi_file, multi_expt):
        logger.info('  Running {}'.format(Analyzer.analysis_name))

        if data_type == 'datam':
            data_dir = self.atmos_datam_dir
        elif data_type == 'dataw':
            data_dir = self.atmos_dataw_dir

        if multi_expt:
            results_dir = os.path.join(self.suite.suite_dir, 'share/data/history/suite_results')
        else:
            results_dir = data_dir[expts[0]]

        analyzer = Analyzer(data_type, data_dir, results_dir, filenames, expts)
        analyzer.set_config(analyzer_config)

        if not analyzer.already_analyzed() or analyzer.force or self.force:
            analyzer.load()
            analyzer.run(self.interactive)
            analyzer.save(self.state, self.suite)
        else:
            logger.info('  Analysis already run')
        return analyzer



import os
import subprocess as sp

import numpy as np

from omnium.omnium_errors import OmniumError


def get_git_info(location):
    cwd = os.getcwd()
    os.chdir(location)
    try:
        git_hash = sp.check_output('git rev-parse HEAD'.split()).strip()
        if sp.check_output('git status --porcelain'.split()) == '':
            return git_hash, 'clean'
        else:
            return git_hash, 'uncommitted_changes'
    except sp.CalledProcessError as ex:
        return None, 'not_git_repo'
    finally:
        os.chdir(cwd)


def is_power_of_two(num):
    return num & (num - 1) == 0


def coarse_grain(data, mask):
    assert data.ndim == 2
    assert data.shape[0] == data.shape[1]
    assert is_power_of_two(data.shape[0])

    nx = data.shape[0]
    pow = int(np.log2(data.shape[0]))

    coarse_data = []
    for n in 2**np.arange(pow + 1):
        coarse = np.zeros((n, n))
        l = nx / n
        for i in range(n):
            s1 = slice(i * l, (i + 1) * l)
            for j in range(n):
                s2 = slice(j * l, (j + 1) * l)
                coarse[i, j] = data[s1, s2][mask[s1, s2]].sum()
        coarse_data.append((n, coarse))
    return coarse_data


def test_indices(i, j, diagonal=False, extended=False):
    if extended:
        # Count any cells in a 5x5 area centred on the current i, j cell as being adjacent.
        indices = []
        for ii in range(i - 2, i + 3):
            for jj in range(j - 2, j + 3):
                indices.append((ii, jj))
    else:
        # Standard, cells sharing a border are adjacent.
        indices = [(i-1, j), (i+1, j), (i, j-1), (i, j+1)]
        if diagonal:
            # Diagonal cells considered adjacent.
            indices += [(i-1, j-1), (i-1, j+1), (i+1, j-1), (i+1, j+1)]
    return indices


def count_blobs_mask(mask, diagonal=False, wrap=True):
    blobs = np.zeros_like(mask, dtype=np.int32)
    blob_index = 0
    for j in range(mask.shape[1]):
        for i in range(mask.shape[0]):
            if blobs[i, j]:
                continue

            if mask[i, j]:
                blob_index += 1
                blobs[i, j] = blob_index
                outers = [(i, j)]
                while outers:
                    new_outers = []
                    for ii, jj in outers:
                        for it, jt in test_indices(ii, jj, diagonal):
                            if not wrap:
                                if it < 0 or it >= mask.shape[0] or\
                                   jt < 0 or jt >= mask.shape[1]:
                                    continue
                            else:
                                it %= mask.shape[0]
                                jt %= mask.shape[1]

                            if not blobs[it, jt] and mask[it, jt]:
                                new_outers.append((it, jt))
                                blobs[it, jt] = blob_index
                    outers = new_outers

    return blob_index, blobs


def get_cube_from_attr(cubes, key, value):
    for cube in cubes:
        if key in cube.attributes:
            if cube.attributes[key] == value:
                return cube
    raise OmniumError('Cube with ({}, {}) not found'.format(key, value))


def get_cube(cubes, section, item):
    for cube in cubes:
        stash = cube.attributes['STASH']
        if stash.section == section and stash.item == item:
            return cube
    raise OmniumError('Cube ({}, {}) not found'.format(section, item))



"""Syncs a mirror with a data source"""
import os
from logging import getLogger

from omnium.omnium_errors import OmniumError
from omnium.syncher import Syncher
from omnium.suite import Suite

logger = getLogger('omnium')

ARGS = [(['--host'], {'help': 'mmuetz@login.archer.ac.uk', 'default': None}),
        (['--create', '-c'], {'help': 'u-AAXXX'}),
        (['--base-path', '-b'], {'help': 'Base path on remote computer',
                                 'default': 'work/cylc-run'}),
        (['--verbose', '-v'], {'help': 'Set verbose mode', 'action': 'store_true'}),
        (['exts'], {'help': 'Additional extensions to include (e.g. .png .pp1.nc)', 'nargs': '*'})]


def main(args):
    suite = Suite()
    try:
        suite.load(os.getcwd())
    except OmniumError:
        pass

    if suite.is_in_suite:
        if args.create:
            logger.error('Cannot use --create from within a suite')
            return 1
        if not suite.is_init:
            logger.error('Suite has not been initialized')
            return 1
        suite_type = suite.suite_config['settings']['suite_type']
        if not suite_type == 'mirror':
            logger.error('Suite is not a mirror, is: {}'.format(suite_type))
            return 1

        suite_name = suite.name
    else:
        if not args.create:
            logger.error('Must be called from within a suite or with --create')
            return 1
        else:
            suite_name = args.create
            if os.path.exists(suite_name):
                logger.error('Mirror {} already exists'.format(suite_name))
                return 1

    if not args.host and args.create:
        host = 'mmuetz@login.archer.ac.uk'
    else:
        host = args.host

    syncher = Syncher(suite, host, args.base_path, args.verbose)
    # Extend with any exts passed by user.
    syncher.add_exts(args.exts)

    if args.create:
        syncher.create(suite_name)
        # create <suite>/.omnium/suite.conf etc.
        suite.load(os.path.join(os.getcwd(), suite_name))
        suite.init('mirror', args.host)
    else:
        syncher.sync()



import datetime as dt

from pyqtgraph.Qt import QtCore, QtGui
import pyqtgraph as pg


class DataDisplayWindow(QtGui.QMainWindow):
    title = '##Data Display##'
    accepts_multiple_cubes = False

    def __init__(self, parent):
        super(DataDisplayWindow, self).__init__(parent)
        self.time_index = 0
        self.level_index = 0
        self.time_days = 0.

    def loadState(self, state):
        self.move(state['pos'])
        self.resize(state['size'])

    def saveState(self):
        state = {'name': self.name}
        state['pos'] = self.pos()
        state['size'] = self.size()
        return state

    def setTime(self, time):
        time_hrs_since_1970 = time
        delta_1970_2000 = dt.datetime(2000, 1, 1) - dt.datetime(1970, 1, 1)
        self.time_days = (dt.timedelta(hours=time_hrs_since_1970) -
                          delta_1970_2000).total_seconds() / 86400.

        for i, test_time in enumerate(self.cube.coord('time').points):
            if test_time == time:
                self.time_index = i
                return
            elif test_time > time:
                if i > 0:
                    self.time_index = i - 1
                else:
                    self.time_index = 0
                return
        self.time_index = self.cube.shape[0] - 1

    def debug(self):
        import ipdb
        ipdb.set_trace()

    def setupGui(self):
        self.menubar = QtGui.QMenuBar(self)
        self.menubar.setGeometry(QtCore.QRect(20, 20, 800, 23))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtGui.QMenu(self.menubar)
        # self.menuFile.setObjectName(_fromUtf8("menuFile"))
        self.menuFile.setTitle("File")
        self.menubar.addAction(self.menuFile.menuAction())
        self.setMenuBar(self.menubar)

        debugAction = QtGui.QAction('&Debug', self)
        debugAction.setShortcut('Ctrl+D')
        debugAction.triggered.connect(self.debug)

        self.menuFile.addAction(debugAction)

        self.setWindowTitle(self.title)
        central_widget = QtGui.QWidget()
        self.main_layout = QtGui.QHBoxLayout()
        central_widget.setLayout(self.main_layout)
        self.setCentralWidget(central_widget)
